function [hc,x,y]=cursorh(action,arg1,NumberOfHarm ,X0,dX)
%[hc,x,y]=cursorh(action,arg1,NumberOfHarm ,X0,dX)
% 
% implement a cursor on a plot
%
%  EXAMPLE:
%		clf reset, plot(rand(10,1)), % create plot
% Num = 3;   ax  = gca;
%		cursorh('init',ax,Num); % create 3 cursors
%		disp(' move cursors,  hit key when done'), pause
%		[X0,dX]=cursorh('read',gca) ;
%		 title(['X0=' num2str(X0) '  dX =' num2str( dX ) blanks(22)]);
%
%
% by I. Bucher 16-1-1995,   bucher@technion.ac.il

global cursorh_co

if nargin<1,
    action='init';
end

if strcmp(action,'init'),
    
    if nargin>1,		% where shell I put the cursorh
        if length(arg1)>0.	% is it cursorh('init',[],...)
            ca=arg1;
        else,
            ca=gca;  
        end
    else,
        ca=gca;			% 
    end
    if nargin<3, NumberOfHarm=1; end
    
    gf=get(ca,'parent');
    replace=strcmp(action,'replace');
    existing=findobj('parent',ca,'makersize',999);
    
    if ~replace & length(existing  )>0,
        errordlg('Only one harmonic cursor per axes is allowed','cursorh');
        return
    elseif length(existing)>0,		% delete old ones
        txt=findobj('userdata',existing);
        others=get(existing,'userdata');
        delete(existing); delete(others); delete(txt);
        clear existing others txt
        action='init';
    end
    
    Xlim=get(ca,'xlim');	DX=diff(Xlim);	Ylim=get(ca,'ylim');
    if nargin<4,  posit=Xlim(1)+0.2376*DX; else, posit=X0;	end
    if nargin>4, DX=dX	; else,  DX=DX/10; end
    if posit>Xlim(2), posit=mean(Xlim); end
    
    HCOLOR= [ 0 1 0];
    
    hc=line('LineStyle','-',...
        'markersize',999,...
        'Xdata',[posit posit],'Ydata',Ylim,...
        'linewidth',2 ,'visible','on',...
        'color',HCOLOR,...
        'userdata',[]);
    
    call=['cursorh(''cursorpress'',0);'];
    set(hc(1),'ButtonDownFcn',call);
    
    
    for q=1:NumberOfHarm,
        hc(q+1)=line('LineStyle','-',...
            'markersize',hc(1),...
            'color',HCOLOR,...
            'Xdata',[posit posit]+q*DX,'Ydata',Ylim,...
            'linewidth',1 ,'visible','on',...
            'userdata',hc(1));
        call=['cursorh(''cursorpress'',' int2str(q) ');'];
        set(hc(q+1),'ButtonDownFcn',call);
    end
    
    set(hc(1),'userdata',hc(2:NumberOfHarm+1));	% store handles for harmonics
    
    %  set-up the nummeric display for X0 and dX
    
    dx=0.07; dy=0.05;
    axpos=get(ca,'pos'); x1=axpos(1)+axpos(3)-dx*4*1.05;; y1=axpos(2)+axpos(4);
    txtPOS=[ x1+(-1)*dx*1.05 y1 dx dy] ;
    
    axpos=get(ca,'pos');
    
    editcall0=['cursorh(''edit'',' int2str(0) ');'];
    
    uicontrol('style','text',...
        'userdata',[hc(1) gca],...
        'unit','normal','pos',[ x1+(0)*dx*1.05 y1 dx dy],...
        'BackgroundColor',[0 .7 1],'ForegroundColor',[1 1 1]*0,...
        'string','x0');
    
    uicontrol('style','edit',...
        'unit','normal',...	
        'userdata',[ca hc],...
        'pos',[ x1+(1)*dx*1.05 y1 dx dy],...  
        'BackgroundColor',[0 0 1],...
        'ForegroundColor',[1 1 1],...
        'callback', editcall0,...
        'string',num2str(posit));
    
    
    uicontrol('style','text',...
        'unit','normal',...	
        'pos',[ x1+(2)*dx*1.05 y1 dx dy],...
        'BackgroundColor',[0 .7 1],'ForegroundColor',[1 1 1]*0,...
        'string', 'Dx');
    
    editcall1=['cursorh(''edit'',' int2str(1) ');'];
    
    uicontrol('style','edit',...
        'unit','normal',...	
        'userdata',[ca hc],...
        'pos',[ x1+(3)*dx*1.05 y1 dx dy],...  
        'BackgroundColor',[0 0 1],...
        'ForegroundColor',[1 1 1],...
        'callback', editcall1,...
        'string',num2str(DX))
    
    
    editcall=['cursorh(''edit'' ,' int2str(1) ' );'];
    
    %	refresh
    
    set(hc,'Erasemode','xor') 
    
elseif strcmp(action,'cursorpress'),
    
    CursorNumber=arg1; 
    call=['cursorh(''cursorpress'',' int2str(CursorNumber) ');'];
    %co= findobj('ButtonDownFcn',call,'parent',gca); 
    co=gco;
    cursorh_co=co;
    set(gcf,'pointer','fleur');
    call=['cursorh(''cursorup'',' int2str(CursorNumber) ');'];
    set(co,'ButtonDownFcn','');
    callmv=['cursorh(''cursormove'' , ' int2str(CursorNumber) ');'];
    set(gcf,'WindowButtonMotionFcn', callmv );
    set(gcf,'WindowButtonUpFcn', call);
    X0=get(co,'xdata'); X0=X0(1);
    set(co,'markersize', abs(X0)+0.5*(X0<0));
    
elseif strcmp(action,'cursormove'),
    global cursorh_co
    CursorNumber=arg1; name=['curharm' int2str(CursorNumber)] ;
    call=['cursorh(''cursorpress'' ,' int2str(CursorNumber) ' );'];
    co= cursorh_co;   %findobj('ButtonDownFcn',call); 
    
    ga=get(co,'parent'); gf=get(ga,'parent');
    % Obtain coordinates of mouse click location in axes units
    pt = get(ga,'Currentpoint');
    t = pt(1,1);
    ax=get(ga,'xlim');
    if t<ax(1),
        t=ax(1);
    elseif t>ax(2),
        t=ax(2);
    end
    
    set(co,'xdata',[t t]);
    
    
elseif strcmp(action,'cursorup'),
    %*******************************>   cursorup
    global cursorh_co
    CursorNumber=arg1;  
    co= cursorh_co;	%  findobj('userdata',name ); 
    if CursorNumber==0;
        h0=co; 	h=get(h0,'userdata');
    else,
        h0=get(co,'userdata'); 	h=get(h0,'userdata');
    end
    
    ca=get(co,'parent'); gf=get(ca,'parent');  
    call=['cursorh(''cursorpress'',' int2str(CursorNumber) ');'];
    set(co ,'ButtonDownFcn',call);
    set(gf,'WindowButtonMotionFcn','');
    set(gf,'WindowButtonUpFcn','');
    set(gf,'pointer','arrow');	
    X0=get(h0,'xdata'); X0=X0(1);
    if CursorNumber>0,	% move one of the harmonics
        Xi=get(co,'xdata'); Xi=Xi(1);	
        dX=(Xi-X0)/CursorNumber;			 
        for q=1:length(h),
            set(h(q),'xdata',(X0+dX*q)*[1 1]);
        end
    else,			% moved the main cursor
        X0old=get(h0,'markersize');
        %		 if fix(X0old)==X0old, X0old=X0old;
        % 		  else, X0old=-fix(X0old); end;
        for q=1:length(h),
            Xq=get(h(q),'xdata'); Xq=Xq(1);
            dX=(Xq-X0old)/q;
            set(h(q),'xdata',(Xq+X0-X0old)*[1 1]);				
        end
    end
    
    data=[ca   h0 h(:)']; 
    editcall0=['cursorh(''edit'',' int2str(0) ');'];
    editcall1=['cursorh(''edit'',' int2str(1) ');'];
    txt1=findobj('userdata',data,'style','edit','callback',editcall0);
    txt2=findobj('userdata',data,'style','edit','callback',editcall1);
    
    set(txt1,'string',num2str(X0));
    set(txt2,'string',num2str(dX));
    set(h0,'markersize',999);
elseif strcmp(action,'edit'),
    %*******************************>   move to edit box,  cursorh('edit',CursorNumber)
    CursorNumber=arg1;
    cotxt=gco;
    cahc=get(cotxt,'userdata');	% get current axis & handles
    hc=cahc(2:length(cahc));	% handles
    ca=cahc(1);			% current axis
    
    editcall0=['cursorh(''edit'',' int2str(0) ');'];
    editcall1=['cursorh(''edit'',' int2str(1) ');'];
    txt1=findobj('userdata',cahc,'style','edit','callback',editcall0);
    txt2=findobj('userdata',cahc,'style','edit','callback',editcall1);
    err=0;
    erroredit='S=''ERR''; err=1;';
    
    S1=get(txt1,'string');
    S2=get(txt2,'string');
    eval(['X0=' S1 ';'], erroredit);
    eval(['dX=' S2 ';'], erroredit);	
    if err,
        set(eval(['txt' int2str(CursorNumber+1)]),'string',S);
    else,
        set(txt1,'string',num2str(X0));
        set(txt2,'string',num2str(dX));
        set(hc(1),'xdata',X0*[1 1]);
        for q=2:length(hc),
            set(hc(q),'xdata',(X0+(q-1)*dX)*[1 1]);				
        end
    end
    
elseif strcmp(action,'read'),
    %*******************************>   [X0,dX]=cursorh('read',ax);
    ax=arg1;		% axes to read from
    call0=['cursorh(''cursorpress'',' int2str(0) ');'];
    call1=['cursorh(''cursorpress'',' int2str(1) ');'];
    h0=findobj('ButtonDownFcn',call0,'parent',ax);
    h=findobj('ButtonDownFcn',call1,'parent',ax);
    if length(h)==0 | length(h0)==0, return, end
    X0=get(h0,'xdata');
    X0dX=get(h(1),'xdata');  
    hc=X0(1); x=X0dX(1)-hc;	% set outputs
    
elseif strcmp(action,'setheight'),
    %*******************************>   setheight
    h=cursorh('gethandle',arg1);
    set(h,'ydata',NumberOfHarm);  % called as cursorh('setheight',gca,[0.01 1])
elseif strcmp(action,'gethandle'),
    %*******************************>   hc=cursorgh('gethandle',gca);
    %				h0=hc(1);  hc(2:N)= harmonics
    ax=arg1;		% axes to read from
    call0=['cursorh(''cursorpress'',' int2str(0) ');'];
    h0=findobj('ButtonDownFcn',call0,'parent',ax);
    if length(h0)==0, hc=[]; return, end
    hc=[h0 get(h0,'userdata')];	% set output
    
elseif strcmp(action,'delete'),
    %*******************************>   delete
    
    
end
